/**********************************************************
                  Header files
 **********************************************************/

#include "lpc214x.h"	
#include "irq.h"		// to use #define __irq __attribute__ ((interrupt("IRQ")))
#include "type.h"		// used for DWORD etc.
#include "stdio.h"		


#define IER_RBR		0x01
#define IER_THRE	0x02
#define IER_RLS		0x04

#define IIR_PEND	0x01
#define IIR_RLS		0x03
#define IIR_RDA		0x02
#define IIR_CTI		0x06
#define IIR_THRE	0x01

#define LSR_RDR		0x01
#define LSR_OE		0x02
#define LSR_PE		0x04
#define LSR_FE		0x08
#define LSR_BI		0x10
#define LSR_THRE	0x20
#define LSR_TEMT	0x40
#define LSR_RXFE	0x80

#define BUFSIZE		0x10
#define TX_BUFSIZE	80



/* *********************************************************
               Function declarations
  ********************************************************* */

void UART1Handler(void) __irq;	// Interrupt-Handler
void Initialize(void);			// PLL-Config
void init_serial1(void);		// Config UART1 
void feed(void);				// PLL - Feed
void init_VIC(void);			// Vectored Interrupt Config

void IRQ_Routine (void)   __attribute__ ((interrupt("IRQ")));	// Normal IRQ-Handler - not used
void FIQ_Routine (void)   __attribute__ ((interrupt("FIQ")));	// not used
void SWI_Routine (void)   __attribute__ ((interrupt("SWI")));	// not used
void UNDEF_Routine (void) __attribute__ ((interrupt("UNDEF")));	// not used


/**********************************************************
                  Global Variables
 **********************************************************/

/**********************************************************
                       MAIN
**********************************************************/

int	main (void) 
{
	
	SCS = 0x03;				// select the "FAST" version of the I/O ports
	FIO1DIR |= 0x00FF0000;	// P1.16...P1.23
	
	Initialize();			// Set PLL
	init_serial1();			// Init UART0 [9600,N,8,1]
	
	U1IER = IER_THRE | IER_RLS | IER_RBR;		// (Re)-enable RBR
	
	init_VIC();
		
	VICVectAddr0	= (unsigned)UART1Handler;	// pass the address of the IRQ into the VIC Slot
	VICVectCntl0 	= 0x00000027; 				// select a priority slot for a given interrupt (UART1 = 27)
	VICIntEnable	= 0x00000080;				// enable interrupt		
	
	FIO1PIN = 0x00010000;						// P1.16 = on (LED) 
	
	while(1)
	{
	
	/* 
	// Polling UART1 - works!
	int ii;
	
	// Start Test Read ADC8 and Display on UART1
		if(U1LSR & 0x01)
		{
			U1THR = U1RBR+2;
		}
	*/
		
		//U1IER = IER_THRE | IER_RLS;			 // Disable RBR
		//U1IER = IER_THRE | IER_RLS | IER_RBR;	// Re-enable RBR
		 
	}
}


void UART1Handler(void)
{
	U1THR = U1RBR;
	FIO1PIN = (0x00000000 + (0x00080000 * 2));// & 0x00FF0000;
	
	VICVectAddr = 0;
}



void init_serial1(void)
{	
	int A = 1, B = 2, C = 3;
	
	// BAUDRATE = 9600 
	// Fpclk	= 15.000.000 (with PLL)
	
	PINSEL0 	= 0x00050000;	// Select UART1 RXD/TXD + EXT1 interrupt
	
	
	//U1FCR		= 0x07;			// Enable and clear FIFO's
	U1LCR		= 0x83;			// 8N1, enable Divisor latch bit
	
	U1DLL 		= 0x47;			// Bei 15MHz
	U1DLM 		= 0x00;
	
	// high precision baudrate-config
	U1LCR		= 0x03;			// Clear DLAB
	U1FDR 		= 0x83;
	U1FCR		= 0x07;			// Enable and clear FIFO's
	
	A = B / B;	// just testing some math
	A = A * B;
}

/**********************************************************
                      Initialize
**********************************************************/
// originally from P.Lynch
#define PLOCK 0x400

void Initialize(void)  {
	   
	// Setting Multiplier and Divider values
  	PLLCFG=0x24;
  	feed();
  
	// Enabling the PLL */	
	PLLCON=0x1;
	feed();
  
	// Wait for the PLL to lock to set frequency
	while(!(PLLSTAT & PLOCK)) ;
  
	// Connect the PLL as the clock source
	PLLCON=0x3;
	feed();
  
	// Enabling MAM and setting number of clocks used for Flash memory fetch (4 cclks in this case)
	MAMCR=0x2;
	MAMTIM=0x4;
  
	// Setting peripheral Clock (pclk) to 1/4 System Clock (cclk)
	VPBDIV=0x0;

}


void feed(void)
{
  PLLFEED=0xAA;
  PLLFEED=0x55;
}

/*  Stubs for various interrupts (may be replaced later)  */
/*  ----------------------------------------------------  */
// originally from P.Lynch
void IRQ_Routine (void) {
	while (1) ;	
}

void FIQ_Routine (void)  {
	while (1) ;	
}


void SWI_Routine (void)  {
	while (1) ;	
}


void UNDEF_Routine (void) {
	while (1) ;	
}

static void DefaultVICHandler (void) // copied from lpc-arm-book-srn.pdf
{
    /* if the IRQ is not installed into the VIC, and interrupt occurs, the
    default interrupt VIC address will be used. This could happen in a race 
    condition. For debugging, use this endless loop to trace back. */
    /* For more details, see Philips appnote AN10414 */
    VICVectAddr = 0;		/* Acknowledge Interrupt */ 
    while ( 1 );
}

void init_VIC(void) // copied from lpc-arm-book-srn.pdf
{
    DWORD i = 0;
    DWORD *vect_addr, *vect_cntl;
   	
    /* initialize VIC*/
    VICIntEnClr = 0xffffffff;
    VICVectAddr = 0;
    VICIntSelect = 0;

    /* set all the vector and vector control register to 0 */
    for ( i = 0; i < VIC_SIZE; i++ )
    {
	vect_addr = (DWORD *)(VIC_BASE_ADDR + VECT_ADDR_INDEX + i*4);
	vect_cntl = (DWORD *)(VIC_BASE_ADDR + VECT_CNTL_INDEX + i*4);
	*vect_addr = 0;	
	*vect_cntl = 0;
    }

    /* Install the default VIC handler here */
    VICDefVectAddr = (DWORD)DefaultVICHandler;   
    return;
}


